package internal

import (
	"context"
	"crypto/sha256"
	"encoding/hex"
	"log"
	"time"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
	"golang.org/x/crypto/bcrypt"
)

func GetMongoClient(url string) *mongo.Client {
	clientOptions := options.Client().ApplyURI(url)

	client, e := mongo.Connect(context.TODO(), clientOptions)
	CheckError(e)

	e = client.Ping(context.TODO(), nil)
	CheckError(e)

	return client
}

func RocketChatRestAdminPass(client *mongo.Client, email, user, pass string) {
	collection := client.Database("rocketchat").Collection("users")

	sha := sha256.Sum256([]byte(pass))
	hexSha := hex.EncodeToString(sha[:])
	hashedPass, err := bcrypt.GenerateFromPassword([]byte(hexSha), bcrypt.DefaultCost)
	CheckError(err)

	filter := bson.D{
		{"$or", bson.A{
			bson.D{{"username", user}},
			bson.D{{"emails.address", email}},
		}},
	}
	update := bson.D{
		{"$set", bson.D{
			{"username", user},
			{"services", bson.D{{"password", bson.D{{"bcrypt", string(hashedPass)}}}}},
		}},
		{"$addToSet", bson.D{
			{"roles", "admin"},
		}},
	}

	r, e := collection.UpdateOne(context.TODO(), filter, update)
	CheckError(e)

	if r != nil && r.MatchedCount > 0 {
		log.Printf("user %s/%s updated", email, user)
	} else {
		log.Printf("user %s/%s not found", email, user)
	}

}

func RocketChatUserCount(client *mongo.Client, active bool, lastLogin int) {
	collection := client.Database("rocketchat").Collection("users")
	filter := bson.D{}
	if active {
		filter = append(filter, primitive.E{"active", true})
	}
	if lastLogin > 0 {
		filter = append(filter, primitive.E{"lastLogin", primitive.D{
			{"$lte", time.Now()},
			{"$gte", time.Now().AddDate(0, 0, -lastLogin)},
		}})
	}

	count, err := collection.CountDocuments(context.TODO(), filter)
	CheckError(err)

	log.Printf("%d users", count)
}
